#include <iostream>
#include <string.h>
using namespace std;

const int MAXS = 100;
const int MAXPAZ = 20;

enum Lab {SANGUE, ELETTRO};
enum Sesso {MASCHIO, FEMMINA};

struct Paziente {
  char nome[MAXS];
  char cognome[MAXS];
  int eta;
  Sesso sex;
};

struct CodaPazienti {
  Paziente paz[MAXPAZ];
  int front, back;
};

struct Ambulatorio {
  CodaPazienti codaSangue;
  CodaPazienti codaElettro;
};

bool full(const CodaPazienti* pc) {
  return (pc->back+1)%MAXPAZ == pc->front;
}

bool insCoda(CodaPazienti* pc, Paziente p) {
  if(full(pc)) return false;
  pc->paz[pc->back] = p;
  pc->back = (pc->back + 1) % MAXPAZ;
  return true;
}

bool inserisci(Ambulatorio* pa, Paziente p, Lab e) {
  if(e == SANGUE) 
    return insCoda(&pa->codaSangue, p);
  else 
    return insCoda(&pa->codaElettro, p);
}

int cerca(const CodaPazienti* pc, const char n[], const char c[]) {
  for(int i=pc->front; i != pc->back; i=(i+1)%MAXPAZ)
    if(strcmp(n, pc->paz[i].nome) == 0 && strcmp(c, pc->paz[i].cognome) == 0)
      return i;
  return -1;
}

void elimina(CodaPazienti* pc, int q) {
  for(int i=q; (i+1) % MAXPAZ != pc->back; i=(i+1)%MAXPAZ)
    pc->paz[i] = pc->paz[(i+1)%MAXPAZ];
  pc->back = pc->back > 0 ? pc->back-1 : MAXPAZ-1;
}

bool desiste(Ambulatorio* pa, const char n[], const char c[]) {
  int q = cerca(&pa->codaSangue, n, c);
  if(q>=0) {
    elimina(&pa->codaSangue, q);
    return true;
  }
  q = cerca(&pa->codaElettro, n, c);
  if(q>=0) {
    elimina(&pa->codaElettro, q);
    return true;
  }
  return false;
}

float mediaCoda(const CodaPazienti* pc) {
  float somma = 0.0f;
  int q = 0;
  for(int i=pc->front; i != pc->back; i=(i+1)%MAXPAZ){
    somma += pc->paz[i].eta;
    q++;
  }
  if(q==0) return 0.0f;
  return somma / q;
}

float etaMedia(const Ambulatorio* pa, Lab e) {
  if(e == SANGUE)
    return mediaCoda(&pa->codaSangue);
  else 
    return mediaCoda(&pa->codaElettro);
}

void ordina(Paziente v[], int n) {
  for(int i=0; i<n-1; i++) {
    int index = i;
    for(int j=i+1; j<n; j++)
      if(strcmp(v[j].cognome, v[index].cognome) < 0) 
        index = j;
    Paziente p = v[index];
    v[index] = v[i];
    v[i] = p;  
  }
}

void stampaOrdinata(const Ambulatorio* pa) {
  Paziente aux[MAXPAZ*2];
  const CodaPazienti* pc = &pa->codaSangue;
  int quanti = 0;
  for(int i=pc->front; i != pc->back; i=(i+1)%MAXPAZ)
    aux[quanti++] = pc->paz[i];
  pc = &pa->codaElettro;
  for(int i=pc->front; i != pc->back; i=(i+1)%MAXPAZ)
    aux[quanti++] = pc->paz[i];
  ordina(aux, quanti);
  for(int i=0; i<quanti; i++)
    cout << aux[i].cognome << '\t' << aux[i].nome << '\t' << aux[i].eta << '\t' << (aux[i].sex==MASCHIO ? 'M' : 'F') << endl;
}

bool empty(const CodaPazienti* pc) {
  return pc->front == pc->back;
}

bool estrai(CodaPazienti* pc) {
  if(empty(pc)) return false;
  pc->front = (pc->front+1)%MAXPAZ;
  return true;
}

bool servizio(Ambulatorio* pa, Lab e) {
  if(e == ELETTRO) 
    if(estrai(&pa->codaElettro))
      return true;
  return estrai(&pa->codaSangue);    
}
